/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * http://www.gnu.org/copyleft/gpl.html
 */
package com.l2dot.gameserver.handler;

import java.util.Map;
import java.util.logging.Logger;

import com.l2dot.Config;
import com.l2dot.gameserver.handler.voicedcommandhandlers.Away;
import com.l2dot.gameserver.handler.voicedcommandhandlers.Banking;
import com.l2dot.gameserver.handler.voicedcommandhandlers.Cl;
import com.l2dot.gameserver.handler.voicedcommandhandlers.OnlinePlayers;
import com.l2dot.gameserver.handler.voicedcommandhandlers.Res;
import com.l2dot.gameserver.handler.voicedcommandhandlers.VoiceInfo;
import com.l2dot.gameserver.handler.voicedcommandhandlers.Wedding;
import com.l2dot.gameserver.handler.voicedcommandhandlers.pmoff;
import com.l2dot.gameserver.handler.voicedcommandhandlers.stat;
import com.l2dot.gameserver.handler.voicedcommandhandlers.stats;
import com.l2dot.gameserver.handler.voicedcommandhandlers.tradeoff;
import com.l2dot.gameserver.handler.voicedcommandhandlers.tvtcmds;
import com.l2dot.gameserver.handler.voicedcommandhandlers.version;
import com.l2dot.gameserver.handler.voicedcommandhandlers.farm1;
import com.l2dot.gameserver.handler.voicedcommandhandlers.farm2;
import com.l2dot.gameserver.handler.voicedcommandhandlers.pvp1;
import com.l2dot.gameserver.handler.voicedcommandhandlers.pvp2;
import com.l2dot.gameserver.handler.voicedcommandhandlers.Command;
import com.l2dot.gameserver.handler.voicedcommandhandlers.Donate;


import javolution.util.FastMap;

/**
 * This class ...
 *
 * @version $Revision: 1.1.4.5 $ $Date: 2005/03/27 15:30:09 $
 */
public class VoicedCommandHandler
{
	private static Logger _log = Logger.getLogger(ItemHandler.class.getName());

	private static VoicedCommandHandler _instance;

	private Map<String, IVoicedCommandHandler> _datatable;

	public static VoicedCommandHandler getInstance()
	{
		if (_instance == null)
		{
			_instance = new VoicedCommandHandler();
		}
		return _instance;
	}

	private VoicedCommandHandler()
	{
		_datatable = new FastMap<String, IVoicedCommandHandler>();
		registerVoicedCommandHandler(new stats());
		if(Config.L2JMOD_ALLOW_WEDDING)
			registerVoicedCommandHandler(new Wedding());
		if(Config.ALLOW_RES_COMMAND)
	      registerVoicedCommandHandler(new Res());  
		  if (Config.ALLOW_AWAY_STATUS) 
			 		                { 
			 	                        registerVoicedCommandHandler(new Away()); 
			 		                } 
		if(Config.ALLOW_TRADEOFF_COMMAND)
	      registerVoicedCommandHandler(new tradeoff());  
		if(Config.ALLOW_CLAN_LEADER_COMMAND)
	      registerVoicedCommandHandler(new Cl());   
		if(Config.ALLOW_INFO_COMMAND)
			registerVoicedCommandHandler(new VoiceInfo());
		if(Config.ALLOW_TVTCMDS_COMMAND)
			registerVoicedCommandHandler(new tvtcmds());		  
		if (Config.ALLOW_STAT_COMMAND)
			registerVoicedCommandHandler(new stat());   
		if (Config.ALLOW_VERSION_COMMAND)
			registerVoicedCommandHandler(new version());
		if(Config.ALLOW_PLAYERS_REFUSAL)
			registerVoicedCommandHandler(new pmoff());
		if(Config.ONLINE_VOICE_ALLOW)
			registerVoicedCommandHandler(new OnlinePlayers());
		if(Config.BANKING_SYSTEM_ENABLED)
			registerVoicedCommandHandler(new Banking());
		if(Config.ALLOW_FARM1_COMMAND)
		    registerVoicedCommandHandler(new farm1());
		if(Config.ALLOW_FARM2_COMMAND)
		    registerVoicedCommandHandler(new farm2());
		if(Config.ALLOW_PVP1_COMMAND)
		    registerVoicedCommandHandler(new pvp1());
		if(Config.ALLOW_PVP2_COMMAND)
		    registerVoicedCommandHandler(new pvp2());
		if(Config.ALLOW_COMMAND_COMMAND)
	        registerVoicedCommandHandler(new Command());
		if(Config.ALLOW_DONATE_COMMAND)
		    registerVoicedCommandHandler(new Donate());
		

		_log.config("VoicedCommandHandler: Loaded " + _datatable.size() + " handlers.");
	}

	public void registerVoicedCommandHandler(IVoicedCommandHandler handler)
	{
		String[] ids = handler.getVoicedCommandList();
		for (int i = 0; i < ids.length; i++)
		{
			if (Config.DEBUG) _log.fine("Adding handler for command "+ids[i]);
			_datatable.put(ids[i], handler);
		}
	}

	public IVoicedCommandHandler getVoicedCommandHandler(String voicedCommand)
	{
		String command = voicedCommand;
		if (voicedCommand.indexOf(" ") != -1) {
			command = voicedCommand.substring(0, voicedCommand.indexOf(" "));
		}
		if (Config.DEBUG)
			_log.fine("getting handler for command: "+command+
					" -> "+(_datatable.get(command) != null));
		return _datatable.get(command);
	}

    /**
     * @return
     */
    public int size()
    {
        return _datatable.size();
    }
}
